<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

use Migrations\AbstractMigration;
use Phinx\Db\Adapter\MysqlAdapter;
use Phinx\Util\Literal;

/**
 * Class Initial
 *
 * Created via:
 * oitc migrations create -p CustomalertModule Initial
 *
 * Run migration:
 * oitc migrations migrate -p CustomalertModule
 *
 */
class Initial extends AbstractMigration {

    /**
     * Whether the tables created in this migration
     * should auto-create an `id` field or not
     *
     * This option is global for all tables created in the migration file.
     * If you set it to false, you have to manually add the primary keys for your
     * tables using the Migrations\Table::addPrimaryKey() method
     *
     * @var bool
     */
    public $autoId = false;

    /**
     * Change Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-change-method
     * @return void
     */
    public function change():void {
        if (!$this->hasTable('customalerts')) {
            $this->table('customalerts')
                ->addColumn('id', 'integer', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 11,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('service_id', 'integer', [
                    'default' => null,
                    'null'    => false,
                ])
                ->addColumn('service_uuid', 'string', [
                    'default' => null,
                    'limit'   => 37,
                    'null'    => false,
                ])
                ->addColumn('message', 'string', [
                    'default' => null,
                    'limit'   => 1024,
                    'null'    => false,
                ])
                ->addColumn('state', 'integer', [
                    'default' => '1',
                    'limit'   => 4,
                    'null'    => true,
                    'signed'  => false,
                ])
                ->addColumn('latest_comment_id', 'integer', [
                    'default' => null,
                    'null'    => true,
                ])
                ->addColumn('created', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addColumn('modified', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addIndex([
                    'service_id'
                ])
                ->addIndex([
                    'service_uuid'
                ])
                ->create();
        }

        if (!$this->hasTable('customalert_comments')) {
            $this->table('customalert_comments')
                ->addColumn('id', 'integer', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 11,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('customalert_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('user_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('comment', 'string', [
                    'default' => null,
                    'limit'   => 1024,
                    'null'    => false,
                ])
                ->addColumn('entry_time', 'datetime', [
                    'timezone' => true,
                    'default' => Literal::from('now()'),
                    'limit'   => null,
                    'null'    => false,
                ])
                ->create();
        }

        if (!$this->hasTable('customalert_statehistory')) {
            $this->table('customalert_statehistory')
                ->addColumn('id', 'integer', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 11,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('customalert_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('state', 'integer', [
                    'default' => '1',
                    'limit'   => 4,
                    'null'    => true,
                    'signed'  => false,
                ])
                ->addColumn('state_time', 'datetime', [
                    'timezone' => true,
                    'default' => Literal::from('now()'),
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addColumn('customalert_comment_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => true,
                ])
                ->create();
        }
    }
}
