<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.


namespace CustomalertModule\Lib\InitialDatabase;

use App\Model\Table\CommandsTable;
use App\Model\Table\ServicetemplatesTable;
use itnovum\openITCOCKPIT\InitialDatabase\Importer;

/**
 * Class TemplateImport
 * @package CustomalertsModule\Lib\InitialDatabase
 */
class TemplateImport extends Importer {
    /**
     * @var CommandsTable
     */
    private $CommandsTable;

    /**
     * @var ServicetemplatesTable
     */
    private $ServicetemplatesTable;

    /**
     * @var array
     */
    private $mapping = [];

    /**
     * TemplateImport constructor.
     * @param CommandsTable $CommandsTable
     * @param ServicetemplatesTable $ServicetemplatesTable
     */
    public function __construct(CommandsTable $CommandsTable, ServicetemplatesTable $ServicetemplatesTable) {
        $this->CommandsTable = $CommandsTable;
        $this->ServicetemplatesTable = $ServicetemplatesTable;
    }

    /**
     * @return bool
     */
    public function import() {
        $data = $this->getData();
        foreach ($data['Commands'] as $command) {
            if (isset($command['uuid']) && !$this->CommandsTable->existsByUuid($command['uuid'])) {
                $entity = $this->CommandsTable->newEntity($command);
                $this->CommandsTable->save($entity);
            }
        }
        $this->mapping['servicetemplates'] = [];

        foreach ($data['Servicetemplates'] as $servicetemplate) {
            if ($this->ServicetemplatesTable->existsByUuid($servicetemplate['uuid'])) {
                $existingServicetemplate = $this->ServicetemplatesTable->getServicetemplateByUuid($servicetemplate['uuid']);
                $this->mapping['servicetemplates'][$servicetemplate['uuid']] = $existingServicetemplate['Servicetemplate']['id'];
                //skip current template if it already exists
                continue;
            }
            if (isset($servicetemplate['uuid']) && !$this->ServicetemplatesTable->existsByUuid($servicetemplate['uuid'])) {
                if (isset($servicetemplate['command_id']) && $this->CommandsTable->existsByUuid($servicetemplate['command_id'])) {
                    $command = $this->CommandsTable->getCommandByUuid($servicetemplate['command_id'], true, false)[0];
                    $servicetemplate['command_id'] = $command['id'];
                    if (!empty($servicetemplate['servicetemplatecommandargumentvalues']) && !empty($command['commandarguments'])) {
                        foreach ($servicetemplate['servicetemplatecommandargumentvalues'] as $templateArgumentKey => $templateArgumentValue) {
                            $servicetemplate['servicetemplatecommandargumentvalues'][$templateArgumentKey] = [
                                'commandargument_id' => $this->getCommandArgumentIdByName($templateArgumentValue['commandargument_id'], $command['commandarguments']),
                                'value'              => $templateArgumentValue['value'],
                            ];
                        }
                    }
                    $entity = $this->ServicetemplatesTable->newEntity($servicetemplate);
                    $this->ServicetemplatesTable->save($entity);
                    $this->mapping['servicetemplates'][$entity->get('uuid')] = $entity->get('id');
                }
            }
        }

        return true;
    }

    /**
     * @return array
     */
    public function getData() {
        return [
            'Commands'         => $this->getCommandsData(),
            'Servicetemplates' => $this->getServicetemplatesData()
        ];
    }

    /**
     * @return array
     */
    public function getCommandsData() {
        $data = [
            0 => [
                'name'             => 'check_customalerts_by_state_and_age',
                'command_line'     => '/opt/openitc/frontend/bin/cake CustomalertModule.check_alerts --statetype 0 -w $ARG1$ -c $ARG2$ --age-warning $ARG3$ --age-critical $ARG4$ --statetype 1 -w $ARG5$ -c $ARG6$ --age-warning $ARG7$ --age-critical $ARG8$ --message "$ARG9$"',
                'command_type'     => CHECK_COMMAND,
                'human_args'       => null,
                'uuid'             => '4cca0317-ce06-4d87-9c04-374d55c2fd37',
                'description'      => 'This plugin is very versatile and can check for custom alerts in different states.
State types: 0=New, 1=In Progress, 2=Done, 3=Manually Closed.
Warning and Critical: Number of alerts before the plugin will output a warning or critical state
Age Warning and Critical: Alerts older than the provided number in days will trigger a warning or critical state',
                'commandarguments' => [
                    [
                        'name'       => '$ARG1$',
                        'human_name' => 'Number of alerts in state "New", before the plugin will output a warning state.',
                    ],
                    [
                        'name'       => '$ARG2$',
                        'human_name' => 'Number of alerts in state "New", before the plugin will output a critical state.',
                    ],
                    [
                        'name'       => '$ARG3$',
                        'human_name' => 'Alerts in state "New" and older than the provided number in days will trigger a warning state.',
                    ],
                    [
                        'name'       => '$ARG4$',
                        'human_name' => 'Alerts in state "New" and older than the provided number in days will trigger a critical state.',
                    ],
                    [
                        'name'       => '$ARG5$',
                        'human_name' => 'Number of alerts in state "In Progress", before the plugin will output a warning state.',
                    ],
                    [
                        'name'       => '$ARG6$',
                        'human_name' => 'Number of alerts in state "In Progress", before the plugin will output a critical state.',
                    ],
                    [
                        'name'       => '$ARG7$',
                        'human_name' => 'Alerts in state "In Progress" and older than the provided number in days will trigger a warning state.',
                    ],
                    [
                        'name'       => '$ARG8$',
                        'human_name' => 'Alerts in state "In Progress" and older than the provided number in days will trigger a critical state.',
                    ],
                    [
                        'name'       => '$ARG9$',
                        'human_name' => 'Only consider alerts where the message matches this string',
                    ],
                ]
            ]

        ];
        return $data;
    }


    /**
     * @return array
     */
    public function getServicetemplatesData() {
        $data = [
            0 => [
                'uuid'                                      => '8e4c2364-b7c5-491d-aee6-7900e6fe3284',
                'template_name'                             => 'CHECK_CUSTOMALERTS_BY_STATE_AND_AGE',
                'name'                                      => 'Custom alerts',
                'container_id'                              => ROOT_CONTAINER,
                'servicetemplatetype_id'                    => GENERIC_SERVICE,
                'check_period_id'                           => '1',
                'notify_period_id'                          => '1',
                'description'                               => '',
                'command_id'                                => '4cca0317-ce06-4d87-9c04-374d55c2fd37',
                'check_command_args'                        => '',
                'checkcommand_info'                         => '',
                'eventhandler_command_id'                   => '0',
                'timeperiod_id'                             => null,
                'check_interval'                            => '300',
                'retry_interval'                            => '60',
                'max_check_attempts'                        => '1',
                'first_notification_delay'                  => '0',
                'notification_interval'                     => '7200',
                'notify_on_warning'                         => '1',
                'notify_on_unknown'                         => '1',
                'notify_on_critical'                        => '1',
                'notify_on_recovery'                        => '1',
                'notify_on_flapping'                        => '0',
                'notify_on_downtime'                        => '0',
                'flap_detection_enabled'                    => '0',
                'flap_detection_on_ok'                      => '0',
                'flap_detection_on_warning'                 => '0',
                'flap_detection_on_unknown'                 => '0',
                'flap_detection_on_critical'                => '0',
                'low_flap_threshold'                        => '0',
                'high_flap_threshold'                       => '0',
                'process_performance_data'                  => '1',
                'freshness_checks_enabled'                  => '0',
                'freshness_threshold'                       => '3600',
                'passive_checks_enabled'                    => '1',
                'event_handler_enabled'                     => '0',
                'active_checks_enabled'                     => '1',
                'retain_status_information'                 => '0',
                'retain_nonstatus_information'              => '0',
                'notifications_enabled'                     => '1',
                'notes'                                     => '',
                'priority'                                  => '1',
                'tags'                                      => '',
                'service_url'                               => '',
                'is_volatile'                               => '0',
                'check_freshness'                           => '0',
                'created'                                   => '2023-03-29 09:54:58',
                'modified'                                  => '2023-03-29 09:54:58',
                'servicetemplateeventcommandargumentvalues' => [],
                'servicetemplatecommandargumentvalues'      => [
                    0 => [
                        'commandargument_id' => '$ARG1$',
                        'value'              => '10',
                    ],
                    1 => [
                        'commandargument_id' => '$ARG2$',
                        'value'              => '20',
                    ],
                    2 => [
                        'commandargument_id' => '$ARG3$',
                        'value'              => '2',
                    ],
                    3 => [
                        'commandargument_id' => '$ARG4$',
                        'value'              => '5',
                    ],
                    4 => [
                        'commandargument_id' => '$ARG5$',
                        'value'              => '25',
                    ],
                    5 => [
                        'commandargument_id' => '$ARG6$',
                        'value'              => '40',
                    ],
                    6 => [
                        'commandargument_id' => '$ARG7$',
                        'value'              => '5',
                    ],
                    7 => [
                        'commandargument_id' => '$ARG8$',
                        'value'              => '10',
                    ],
                    8 => [
                        'commandargument_id' => '$ARG9$',
                        'value'              => '',
                    ],
                ],
                'customvariables'                           => [],
                'servicegroups'                             => [],
                'contactgroups'                             => [],
                'contacts'                                  => [],
            ],
        ];
        return $data;
    }


    /**
     * @param string $name
     * @param array $commandarguments
     * @return bool|mixed
     */
    public function getCommandArgumentIdByName(string $name, array $commandarguments) {
        foreach ($commandarguments as $commandargument) {
            if (isset($commandargument['name']) && $commandargument['name'] === $name) {
                return $commandargument['id'];
            }
        }
        return false;
    }
}
