<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace CustomalertModule\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use CustomalertModule\Model\Entity\CustomalertComment;

/**
 * CustomalertComments Model
 *
 * @property CustomalertsTable&\Cake\ORM\Association\BelongsTo $Customalerts
 * @property UsersTable&\Cake\ORM\Association\BelongsTo $Users
 * @property CustomalertStatehistoryTable&\Cake\ORM\Association\HasMany $CustomalertStatehistory
 *
 * @method CustomalertComment newEmptyEntity()
 * @method CustomalertComment newEntity(array $data, array $options = [])
 * @method CustomalertComment[] newEntities(array $data, array $options = [])
 * @method CustomalertComment get($primaryKey, $options = [])
 * @method CustomalertComment findOrCreate($search, ?callable $callback = null, $options = [])
 * @method CustomalertComment patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method CustomalertComment[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method CustomalertComment|false save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method CustomalertComment saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method CustomalertComment[]|\Cake\Datasource\ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method CustomalertComment[]|\Cake\Datasource\ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method CustomalertComment[]|\Cake\Datasource\ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method CustomalertComment[]|\Cake\Datasource\ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 */
class CustomalertCommentsTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('customalert_comments');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsTo('Customalerts', [
            'foreignKey' => 'customalert_id',
            'joinType'   => 'INNER',
            'className'  => 'CustomalertModule.Customalerts',
        ]);
        $this->belongsTo('Users', [
            'foreignKey' => 'user_id',
            'joinType'   => 'LEFT',
            'className'  => 'Users',
        ]);
        $this->hasMany('CustomalertStatehistory', [
            'foreignKey' => 'customalert_comment_id',
            'className'  => 'CustomalertModule.CustomalertStatehistory',
        ]);
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator {
        $validator
            ->integer('customalert_id')
            ->requirePresence('customalert_id', 'create')
            ->notEmptyString('customalert_id');

        $validator
            ->integer('user_id')
            ->requirePresence('user_id', 'create')
            ->notEmptyString('user_id');

        $validator
            ->scalar('comment')
            ->maxLength('comment', 1024)
            ->requirePresence('comment', 'create')
            ->notEmptyString('comment');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param RulesChecker $rules The rules object to be modified.
     * @return RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker {
        $rules->add($rules->existsIn('customalert_id', 'Customalerts'), ['errorField' => 'customalert_id']);
        //$rules->add($rules->existsIn('user_id', 'Users'), ['errorField' => 'user_id']);

        return $rules;
    }
}
