<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace CustomalertModule\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use CustomalertModule\Model\Entity\CustomalertStatehistory;

/**
 * CustomalertStatehistory Model
 *
 * @property CustomalertsTable&\Cake\ORM\Association\BelongsTo $Customalerts
 * @property CustomalertCommentsTable&\Cake\ORM\Association\BelongsTo $CustomalertComments
 *
 * @method CustomalertStatehistory newEmptyEntity()
 * @method CustomalertStatehistory newEntity(array $data, array $options = [])
 * @method CustomalertStatehistory[] newEntities(array $data, array $options = [])
 * @method CustomalertStatehistory get($primaryKey, $options = [])
 * @method CustomalertStatehistory findOrCreate($search, ?callable $callback = null, $options = [])
 * @method CustomalertStatehistory patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method CustomalertStatehistory[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method CustomalertStatehistory|false save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method CustomalertStatehistory saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method CustomalertStatehistory[]|\Cake\Datasource\ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method CustomalertStatehistory[]|\Cake\Datasource\ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method CustomalertStatehistory[]|\Cake\Datasource\ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method CustomalertStatehistory[]|\Cake\Datasource\ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 */
class CustomalertStatehistoryTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('customalert_statehistory');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->belongsTo('Customalerts', [
            'foreignKey' => 'customalert_id',
            'joinType'   => 'INNER',
            'className'  => 'CustomalertModule.Customalerts',
        ]);
        $this->belongsTo('CustomalertComments', [
            'foreignKey' => 'customalert_comment_id',
            'className'  => 'CustomalertModule.CustomalertComments',
        ]);
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator {
        $validator
            ->integer('customalert_id')
            ->requirePresence('customalert_id', 'create')
            ->notEmptyString('customalert_id');

        $validator
            ->nonNegativeInteger('state')
            ->allowEmptyString('state');

        $validator
            ->dateTime('state_time')
            ->requirePresence('state_time', 'create')
            ->notEmptyDateTime('state_time');

        $validator
            ->integer('customalert_comment_id')
            ->allowEmptyString('customalert_comment_id');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param RulesChecker $rules The rules object to be modified.
     * @return RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker {
        $rules->add($rules->existsIn('customalert_id', 'Customalerts'), ['errorField' => 'customalert_id']);
        $rules->add($rules->existsIn('customalert_comment_id', 'CustomalertComments'), ['errorField' => 'customalert_comment_id']);

        return $rules;
    }
}
