#!/bin/bash

# Class named 'CustomalertModule' for UPDATE.sh bash object emulation

# property
properties=()

# properties IDs (only placeholder with incrementing array position value)
name=0
disableFileLock=1

# predefined properties
properties[name]='CustomalertModule'
# set 1 to disable file lock, this disables the uninstall() method and will run install() on every openitcockpit-update if the module folder exists
properties[disableFileLock]=1



# do not change this method!
CustomalertModule.property(){
    if [ "$2" == "=" ]; then
        properties[$1]=$3
    else
        echo ${properties[$1]}
    fi
}

# runs on every openitcockpit-update
CustomalertModule.initialize(){
    CustomalertModule.installIfNeeded
}

CustomalertModule.installIfNeeded(){
    if [ $(CustomalertModule.property disableFileLock) == 1 ]; then
        if [ $(system.moduleSrcDirExists $(CustomalertModule.property name)) == 1 ]; then
            CustomalertModule.install
        fi
    else
        if [ $(system.moduleSrcDirExists $(CustomalertModule.property name)) == 1 ]; then
            if [ "$(system.isLocked $(CustomalertModule.property name) 'installed')" != 1 ]; then
                CustomalertModule.install
            fi
        else
            if [ "$(system.isLocked $(CustomalertModule.property name) 'installed')" == 1 ]; then
                CustomalertModule.uninstall
            fi
        fi
    fi
}

CustomalertModule.install(){
    echo "Run $(CustomalertModule.property name) update"
    cp -r /opt/openitc/frontend/plugins/$(CustomalertModule.property name)/system/lib/. /lib/
    systemctl daemon-reload

    system.lock $(CustomalertModule.property name) "installed" $(CustomalertModule.property disableFileLock)
}

CustomalertModule.uninstall(){
    echo "I am the migrated postrm routine of $(CustomalertModule.property name)"

    systemctl stop customalert_worker.service
    systemctl disable customalert_worker.service

    rm -rf /lib/systemd/system/customalert_worker.service

    # insert custom code
    system.unlock $(CustomalertModule.property name) "installed" $(CustomalertModule.property disableFileLock)
}

CustomalertModule.finish(){
    if [ $(system.moduleSrcDirExists $(CustomalertModule.property name)) == 1 ]; then
        echo "Detected CustomalertModule: try to restart CustomAlertWorker"
        systemctl enable customalert_worker.service
        systemctl restart customalert_worker.service
    fi
}
