angular.module('openITCOCKPIT')
    .controller('CustomalertsIndexController', function($scope, $http, SortService, MassChangeService){

        SortService.setSort('Customalerts.created');
        SortService.setDirection('desc');

        $scope.currentPage = 1;
        $scope.useScroll = true;

        $scope.massChange = {};
        $scope.selectedElements = 0;

        $scope.annotateUrl = '/customalert_module/customalerts/annotate/';
        $scope.closeUrl = '/customalert_module/customalerts/close/';
        $scope.CustomalertComment = {
            comment: null
        };

        $scope.groupedView = 0;

        var now = new Date();

        /*** Filter Settings ***/
        var defaultFilter = function(){
            $scope.filter = {
                Customalerts: {
                    message: '',
                    state: {
                        0: true,
                        1: true,
                        2: false,
                        3: false
                    }
                },
                Hosts: {
                    container_id: []
                },
                from: date('d.m.Y H:i', now.getTime() / 1000 - (3600 * 24 * 30 * 6)), // last 6 months
                to: date('d.m.Y H:i', now.getTime() / 1000 + (3600 * 24)),
                recursive: false
            };
            var from = new Date(now.getTime() - (3600 * 24 * 30 * 6 * 1000)); // last 6 months
            from.setSeconds(0);
            var to = new Date(now.getTime() + (3600 * 24 * 1000));
            to.setSeconds(0);
            $scope.from_time = from;
            $scope.to_time = to;
        };
        /*** Filter end ***/
        $scope.init = true;
        $scope.showFilter = false;

        var groupByHost = function(){
            $scope.customalertsGrouped = [];

            var customAlertsFormated = {};
            var keepOrder = [];
            for(var i in $scope.customalerts){
                var customalert = $scope.customalerts[i];
                if(!customAlertsFormated.hasOwnProperty(customalert.service.host_id)){
                    // Push the host_ids into an array to restore the order from the server
                    keepOrder.push(customalert.service.host_id);

                    customAlertsFormated[customalert.service.host_id] = {
                        host: {
                            id: customalert.service.host_id,
                            name: customalert.service.host.name,
                            primary_container: customalert.service.primary_container
                        },
                        customalerts: []
                    }
                }
                customAlertsFormated[customalert.service.host_id].customalerts.push(customalert);
            }

            keepOrder.forEach(function(hostId){
                $scope.customalertsGrouped.push(customAlertsFormated[hostId]);
            });
        };

        $scope.loadContainers = function(){
            $http.get("/customalert_module/customalerts/loadContainers.json", {
                params: {
                    'angular': true
                }
            }).then(function(result){
                $scope.containers = result.data.containers;
            });
        };

        $scope.load = function(){
            var stateFilter = [];
            for(var key in $scope.filter.Customalerts.state){
                if($scope.filter.Customalerts.state[key] === true){
                    stateFilter.push(key);
                }
            }
            var params = {
                'angular': true,
                'scroll': $scope.useScroll,
                'sort': SortService.getSort(),
                'page': $scope.currentPage,
                'direction': SortService.getDirection(),
                'recursive': $scope.filter.recursive,
                'filter[Customalerts.message]': $scope.filter.Customalerts.message,
                'filter[Customalerts.state][]': stateFilter,
                'filter[Hosts.container_id][]': $scope.filter.Hosts.container_id,
                'filter[from]': $scope.filter.from,
                'filter[to]': $scope.filter.to
            };
            $http.get("/customalert_module/customalerts/index.json", {
                    params: params
                }
            ).then(function(result){
                $scope.customalerts = result.data.customalerts;
                $scope.customalertGlobalOverview = result.data.customalertGlobalOverview;

                if($scope.groupedView){
                    groupByHost();
                }
                $scope.paging = result.data.paging;
                $scope.scroll = result.data.scroll;
                $scope.init = false;
            });
        };

        $scope.triggerFilter = function(){
            $scope.showFilter = !$scope.showFilter === true;
        };

        $scope.resetFilter = function(){
            defaultFilter();
            $scope.undoSelection();
        };

        $scope.changepage = function(page){
            if(page !== $scope.currentPage){
                $scope.currentPage = page;
                $scope.load();
            }
        };

        $scope.changeMode = function(val){
            $scope.useScroll = val;
            $scope.load();
        };

        $scope.undoSelection = function(){
            MassChangeService.clearSelection();
            $scope.massChange = MassChangeService.getSelected();
            $scope.selectedElements = MassChangeService.getCount();
        };

        $scope.selectAll = function(){
            if($scope.customalerts){
                for(var key in $scope.customalerts){
                    if($scope.customalerts[key].allow_edit){
                        var id = $scope.customalerts[key].id;
                        $scope.massChange[id] = true;
                    }
                }
            }
        };

        $scope.getObjectsForAnnotate = function(){
            var objects = {};
            var selectedObjects = MassChangeService.getSelected();
            for(var key in $scope.customalerts){
                for(var id in selectedObjects){
                    if(id == $scope.customalerts[key].id){
                        objects[id] = {
                            name: $scope.customalerts[key].service.host.name + '/' + $scope.customalerts[key].servicename,
                            host_id: $scope.customalerts[key].service.host_id,
                            service_id: $scope.customalerts[key].service_id,
                        }
                    }
                }
            }
            return objects;
        };

        $scope.getObjectForAnnotate = function(customalert){
            var object = {};
            object[customalert.id] = {
                name: customalert.service.host.name + '/' + customalert.servicename,
                host_id: customalert.service.host_id,
                service_id: customalert.service_id,
            }
            return object;
        };

        $scope.getObjectsForClose = function(){
            var objects = {};
            var selectedObjects = MassChangeService.getSelected();

            for(var key in $scope.customalerts){
                for(var id in selectedObjects){
                    // only services with state new => 0 and in progress => 1 can be manually closed
                    if(id == $scope.customalerts[key].id && $scope.customalerts[key].state < 2){
                        objects[id] = $scope.customalerts[key].service.host.name + '/' + $scope.customalerts[key].servicename;
                    }
                }
            }
            return objects;
        };

        $scope.getObjectForClose = function(customalert){
            var object = {};
            object[customalert.id] = customalert.service.host.name + '/' + customalert.servicename;
            return object;
        };

        $scope.triggerLoadByBookmark = function(filter){
            if(typeof filter !== "undefined"){
                $scope.init = true; //Disable $watch to avoid two HTTP requests

                // Reset from and to with default values - makes so sens to store this
                filter.from = date('d.m.Y H:i', now.getTime() / 1000 - (3600 * 24 * 30));
                filter.to = date('d.m.Y H:i', now.getTime() / 1000 + (3600 * 24));

                // Reset date input fields
                var from = new Date(now.getTime() - (3600 * 24 * 30 * 1000));
                from.setSeconds(0);
                var to = new Date(now.getTime() + (3600 * 24 * 1000));
                to.setSeconds(0);
                $scope.from_time = from;
                $scope.to_time = to;

                $scope.filter = filter;
            }else{
                $scope.init = true;
                $scope.resetFilter();
            }

            $scope.currentPage = 1;
            $scope.undoSelection();
            $scope.load();
        }

        //Fire on page load
        defaultFilter();
        $scope.loadContainers();
        SortService.setCallback($scope.load);

        $scope.$watch('filter', function(){
            $scope.currentPage = 1;
            $scope.load();
        }, true);

        $scope.$watch('massChange', function(){
            MassChangeService.setSelected($scope.massChange);
            $scope.selectedElements = MassChangeService.getCount();
        }, true);

        $scope.$watch('groupedView', function(){
            if($scope.init){
                return;
            }

            if($scope.groupedView){
                groupByHost();
            }

        }, true);


        $scope.$watch('from_time', function(dateObject){
            if(dateObject !== undefined && dateObject instanceof Date){
                var dateString = date('d.m.Y H:i', dateObject.getTime() / 1000);
                $scope.filter.from = dateString;
            }
        });
        $scope.$watch('to_time', function(dateObject){
            if(dateObject !== undefined && dateObject instanceof Date){
                var dateString = date('d.m.Y H:i', dateObject.getTime() / 1000);
                $scope.filter.to = dateString;
            }
        });

    });
